import scrapy


class QuotesSpider(scrapy.Spider):
    name = 'BrainyQuote'
    base_url = 'https://www.brainyquote.com'
    start_urls = [
        base_url+'/quote_of_the_day',
    ]

    def parse(self, response):
        for quote in response.css('div.qll-bg'):
            tags = []
            for tag in quote.css('a.qkw-btn'):
                tags.append(tag.xpath('text()').get())
            yield {
                'author': quote.xpath('div/div/div/a/text()').get(),
                'text': quote.xpath('div/div/a/text()').get(),
                'url': self.base_url+quote.xpath('div/div/a/@href').get(),
                'tags': tags
            }



import io
import json
import random
import logging
from importlib_metadata import version, PackageNotFoundError
import click
from scrapy.crawler import CrawlerProcess
from rich.console import Console
from rich.logging import RichHandler
# spiders
from quoted.scrapy.spiders import toscrape, brainyquote, goodreads

# Logging
logger = logging.getLogger(__name__)

# Buffer
bytestream = io.BytesIO()


def init_logging(log_level=logging.CRITICAL):
    FORMAT = "%(message)s"
    logging.basicConfig(
        level=log_level,
        format=FORMAT, datefmt="[%X] ",
        handlers=[RichHandler(level=log_level)]
    )

    return logging.getLogger(__name__)


def get_spider():
    spiders = [
        toscrape.QuotesSpider,
        brainyquote.QuotesSpider,
        goodreads.QuotesSpider
    ]
    spider_selector = random.randint(1, len(spiders)-1)

    return spiders[spider_selector]


def do_crawl(spider):
    """
    Crawl web sites to get quotes using scrapy spiders.
    module: quoted.quoted.spiders
    """
    process = CrawlerProcess(settings={
        "LOG_ENABLED": False,
        "TELNETCONSOLE_ENABLED": False,
        "FEED_STORAGES": {
            'buffered': 'quoted.scrapy.extensions.storage.BufferedFeedStorage',
        },
        "FEED_STORAGE_BUFFERED": {
            "bytestream": {
                "module": "quoted.quoted",
                "buffer": "bytestream"
            },
        },
        "FEEDS": {
            "buffered:bytestream": {"format": "json"},
        },
    })

    process.crawl(spider)

    # the script will block here until the crawling is finished
    process.start()


def get_quote_from_json_stream(stream):
    stream_value = stream.getvalue()
    logger.debug(stream_value)
    quotes = json.loads(stream_value)
    logger.debug(quotes)
    quote_selector = random.randint(1, len(quotes)-1)

    return quotes[quote_selector]


@click.command()
@click.option(
    '--rich-text/--no-rich-text',
    ' /-R',
    help='Rich Text support',
    default=True
)
@click.option(
    '--show-tags/--no-show-tags',
    ' /-T',
    help='Show or Hide quote tags',
    default=True
)
@click.option(
    '--show-link/--no-show-link',
    ' /-L',
    help='Show or Hide quote link',
    default=True
)
@click.option(
    '--log-level',
    '-l',
    help='Set log level',
    type=click.Choice(
        [
            'CRITICAL',
            'ERROR',
            'WARNING',
            'INFO',
            'DEBUG',
            'NOTSET'
        ],
        case_sensitive=False
    ),
    default='CRITICAL'
)
@click.option(
    '--version',
    '-v',
    'show_version',
    help='Print version information and quit',
    is_flag=True
)
def main(
    rich_text=True,
    show_tags=True,
    show_link=True,
    log_level="CRITICAL",
    show_version=False
):
    """Feed your brain with the best random quotes from multiple web portals"""
    print_styles = {
        "text": "italic",
        "author": "bold"
    }

    if not rich_text:
        print_styles = {
            "text": "",
            "author": ""
        }

    logger = init_logging(log_level)
    console = Console()

    if show_version:
        quoted_version = 'master'
        try:
            quoted_version = version('quoted')
        except PackageNotFoundError:
            logger.debug("PackageNotFoundError: quoted version not found")
        console.print('Quoted version %s' % quoted_version)
        exit()

    spider = get_spider()
    do_crawl(spider)

    try:
        quote = get_quote_from_json_stream(bytestream)

        console.print("")
        console.print(
            "“%s”" % quote["text"],
            style=print_styles["text"],
            highlight=False
        )
        console.print("―― %s" % quote["author"], style=print_styles["author"])
        console.print("")
        if show_tags:
            console.print("tags: %s" % ', '.join(quote["tags"]))
        if show_link:
            console.print("link: %s" % quote["url"])
        if show_tags or show_link:
            console.print("")
        console.print("© %s" % spider.name)
        console.print("")
        console.print("Powered by quoted")

    except json.JSONDecodeError:
        logger.error("JSONDecodeError: Failed parsing json response!")
    except TypeError:
        logger.error("TypeError: Failed parsing json response!")


if __name__ == "__main__":
    main()



import scrapy
import random


class QuotesSpider(scrapy.Spider):
    name = 'goodreads'
    base_url = 'https://www.goodreads.com'
    page = random.randint(1, 100)

    start_urls = [
        base_url+'/quotes?page='+str(page),
    ]

    def parse(self, response):
        for quote in response.css('div.quote'):
            tags = []
            for tag in quote.css('div.greyText > a'):
                tags.append(tag.xpath('text()').get())

            author = quote.css('span.authorOrTitle::text')
            text = quote.css('div.quoteText::text')
            url = quote.css('a.smallText::attr("href")')

            yield {
                'author': author.get().strip(', \n\r\t“”'),
                'text': text.get().strip(', \n\r\t“”'),
                'url': self.base_url+url.get().strip(),
                'tags': tags
            }



import scrapy


class QuotesSpider(scrapy.Spider):
    name = 'Quotes to Scrape'
    start_urls = [
        'http://quotes.toscrape.com/tag/humor/',
    ]

    def parse(self, response):
        for quote in response.css('div.quote'):
            yield {
                'author': quote.xpath('span/small/text()').get(),
                'text': quote.css('span.text::text').get(),
            }

        next_page = response.css('li.next a::attr("href")').get()
        if next_page is not None:
            yield response.follow(next_page, self.parse)



import logging
import sys
from urllib.parse import urlparse
from zope.interface import implementer
from scrapy.extensions.feedexport import IFeedStorage


logger = logging.getLogger(__name__)


@implementer(IFeedStorage)
class BufferedFeedStorage:
    """
    Feeds are written to the buffer like `io.BytesIO` or `sys.stdout.buffer`
    identified by key on uri.
    - URI scheme: buffered
    - Example URI: buffered:bytestream
      Where bytestream is the buffered key, this need match with key on
      FEED_STORAGE_BUFFERED
    - Required external libraries: none

    The buffer options are configurated on FEED_STORAGE_BUFFERED var.
    Example:
    "FEED_STORAGE_BUFFERED": {
            "bytestream": {
                "module": "quoted.quoted",
                "buffer": "bytestream"
                },
        }

    - `bytestream` key need match with id of uri (buffered:key)
    - `module` is the python module where var is located
    - `buffer` is the name of var to use
    """

    def __init__(self, uri):
        self._buffer_id = urlparse(uri).path
        self._buffer = None
        self._module_name = None
        self._buffer_name = None

    def open(self, spider):
        settings = spider.crawler.settings['FEED_STORAGE_BUFFERED']

        if self._buffer_id not in settings:
            logger.error(
                "Buffer id %s not found in FEED_STORAGE_BUFFERED settings"
                %
                (self._buffer_id)
            )
            return self._buffer_id

        buffer_settings = settings[self._buffer_id]
        self._module_name = buffer_settings["module"]
        logger.debug("Module Name: %s" % (self._module_name))

        module_object = sys.modules[self._module_name]
        logger.debug("Module Object: %s" % (str(module_object)))

        self._buffer_name = buffer_settings["buffer"]
        logger.debug("Buffer Name: %s" % (self._buffer_name))

        self._buffer = getattr(module_object, self._buffer_name)
        logger.debug("Buffer Object: %s" % (str(self._buffer)))

        return self._buffer

    def store(self, file):
        module_object = sys.modules[self._module_name]
        # update object reference to file argument
        setattr(module_object, self._buffer_name, file)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['quoted',
 'quoted.scrapy',
 'quoted.scrapy.extensions',
 'quoted.scrapy.spiders']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'importlib-metadata>=1.6.0,<2.0.0',
 'rich>=1.0.3,<2.0.0',
 'scrapy>=2.1.0,<3.0.0']

entry_points = \
{'console_scripts': ['quoted = quoted.quoted:main']}

setup_kwargs = {
    'name': 'quoted',
    'version': '0.3.0',
    'description': 'Feed your brain with the best random quotes from multiple web portals.',
    'long_description': "# quoted\n\nFeed your brain with the best random quotes from multiple web portals.\n\n## Features\n\n* Multiple WEB sources\n* Rich Text\n* Argument options\n* Logs\n\n## Requirements\n\n```\ngit\npython 3x\npoetry\n```\n\n## Installation\n\n### Linux/MacOS\n\n```\n$ pip install quoted\n```\n\n### Windows\n\n\n## Usage\n\n```\n$ quoted\n\n\xe2\x80\x9cInsanity is doing the same thing, over and over again, but expecting different results.\xe2\x80\x9d\n\xe2\x80\x95\xe2\x80\x95 Narcotics Anonymous\n\ntags: humor, insanity, life, misattributed-ben-franklin, misattributed-mark-twain, misattributed-to-einstein\nlink: https://www.goodreads.com/quotes/5543-insanity-is-doing-the-same-thing-over-and-over-again\n\n\xc2\xa9 goodreads\n\nPowered by quoted\n```\n## Development\n\n### Run\n\n```\n$ poetry install\n$ poetry run quoted\n```\n\n### Build\n\n```\n$ poetry build\n```\n\nThe distribution packages are located in `dist` directory.\n\n### Publish\n\n```\n$ poetry publish\n```\n\n### Spiders\n\nSpider output is a list of dicts with the structure:\n```\n[\n    {\n        'author': 'Author Name',\n        'text': 'Text of Quote',\n        'tags': ['tag1','tag2'],\n        'url': 'https://www.quotesource.com/linktoquote'\n    }\n]\n```\n\n## Todo\n\n* Cache\n* Supports `bash` and `zsh`\n* Output formats\n\n## Contribution\n\n* File bugs, feature requests in [GitHub Issues](https://github.com/rcares/quoted/issues).\n",
    'author': 'Rodrigo Cares',
    'author_email': 'rcares@gmail.com',
    'maintainer': 'Rodrigo Cares',
    'maintainer_email': 'rcares@gmail.com',
    'url': 'https://github.com/rcares/quoted/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
